@@ -266,47 +266,51 @@ async def bio_skip_bytes_async(
 
 def bytes_iter_to_reader(it: Iterable[bytes | bytearray], /) -> SupportsRead[bytes]:
     get_next = iter(it).__next__
-    unconsumed = bytearray(b"")
+    at_eof = False
+    unconsumed: bytearray = bytearray(b"")
     def read(n=-1):
-        nonlocal unconsumed
-        if n == 0:
+        nonlocal at_eof, unconsumed
+        if at_eof or n == 0:
             return b""
         try:
             if n < 0:
                 while True:
-                    unconsumed.extend(get_next())
+                    unconsumed += get_next()
             else:
                 while n > len(unconsumed):
-                    unconsumed.extend(get_next())
+                    unconsumed += get_next()
                 b, unconsumed = unconsumed[:n], unconsumed[n:]
                 return bytes(b)
         except StopIteration:
+            at_eof = True
             return bytes(unconsumed)
     reprs = f"<reader for {it!r}>"
     return type("reader", (), {"read": staticmethod(read), "__repr__": staticmethod(lambda: reprs)})()
 
 
 def bytes_iter_to_reader_async(it: Iterable[bytes | bytearray] | AsyncIterable[bytes | bytearray], /) -> SupportsRead[bytes]:
-    unconsumed = bytearray(b"")
     if isinstance(it, AsyncIterable):
         get_next = aiter(it).__anext__
     else:
         sync_next = iter(it).__next__
         get_next = lambda: to_thread(sync_next)
+    at_eof = False
+    unconsumed: bytearray = bytearray(b"")
     async def read(n=-1):
-        nonlocal unconsumed
-        if n == 0:
+        nonlocal at_eof, unconsumed
+        if at_eof or n == 0:
             return b""
         try:
             if n < 0:
                 while True:
-                    unconsumed.extend(await get_next())
+                    unconsumed += await get_next()
             else:
                 while n > len(unconsumed):
-                    unconsumed.extend(await get_next())
+                    unconsumed += await get_next()
                 b, unconsumed = unconsumed[:n], unconsumed[n:]
                 return bytes(b)
         except StopIteration:
+            at_eof = True
             return bytes(unconsumed)
     reprs = f"<reader for {it!r}>"
     return type("reader", (), {"read": staticmethod(read), "__repr__": staticmethod(lambda: reprs)})()
